@@ -3,11 +3,11 @@
 #
 # SoundConverter - GNOME application for converting between audio formats. 
 # Copyright 2004 Lars Wirzenius
-# Copyright 2005-2007 Gautier Portet
+# Copyright 2005-2008 Gautier Portet
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; version 2 of the License.
+# the Free Software Foundation; version 3 of the License.
 #
 # This program is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -36,6 +36,7 @@ import string
 import thread
 import urllib
 import time
+import unicodedata
 
 # GNOME and related stuff.
 try:
@@ -77,7 +78,7 @@ PACKAGE = NAME.lower()
 gettext.bindtextdomain(PACKAGE,"@datadir@/locale")
 locale.setlocale(locale.LC_ALL,"")
 gettext.textdomain(PACKAGE)
-gettext.install(PACKAGE,localedir=None,unicode=1)
+gettext.install(PACKAGE,localedir="@datadir@/locale",unicode=1)
 
 gtk.glade.bindtextdomain(PACKAGE,"@datadir@/locale")
 gtk.glade.textdomain(PACKAGE)
@@ -89,6 +90,14 @@ Jonh Wendell <wendell bani.com.br> (Portuguese Brazilian)
 Marc E. <m4rccd yahoo.com> (Spanish)
 Daniel Nylander <po danielnylander se> (Swedish)
 Alexandre Prokoudine <alexandre.prokoudine gmail.com> (Russian) 
+Kamil Páral <ripper42 gmail.com > (Czech)
+Stefano Luciani <luciani.fa tiscali.it > (Italian)
+Martin Seifert <martinseifert fastmail.fm> (German)
+Nizar Kerkeni <nizar.kerkeni gmail.com>(Arabic)
+amenudo (Basque)
+rainofchaos (Simplified Chinese)
+Pavol Klačanský (Slovak)
+Moshe Basanchig <moshe.basanchig gmail.com> (Hebrew)
 """)
 
 # Names of columns in the file list
@@ -106,8 +115,37 @@ mime_whitelist = (
 	"application/x-ape",
 	"application/vnd.rn-realmedia",
 	"application/x-shockwave-flash",
+	"application/x-3gp",
 )
 
+# custom filename patterns
+english_patterns = "Artist Album Title Track Total Genre Date Year"
+
+# traductors: only if it does make sense.
+locale_patterns = _("Artist Album Title Track Total Genre Date Year")
+
+patterns_formats = (
+	"%(artist)s",
+	"%(album)s",
+	"%(title)s",
+	"%(track-number)02d",
+	"%(track-count)02d",
+	"%(genre)s",
+	"%(date)s",
+	"%(year)s",
+)
+
+# add english and locale
+custom_patterns = english_patterns + " " + locale_patterns
+# convert to list
+custom_patterns = [ "{%s}" % p for p in custom_patterns.split()]
+# and finally to dict, thus removing doubles 
+custom_patterns = dict(zip(custom_patterns, patterns_formats*2))
+
+locale_patterns_dict = dict(zip(
+	[ p.lower() for p in english_patterns.split()],
+	[ "{%s}" % p for p in locale_patterns.split()] ))
+
 # add here the formats not containing tags 
 # not to bother searching in them
 tag_blacklist = (
@@ -118,15 +156,21 @@ tag_blacklist = (
 # Name and pattern for CustomFileChooser
 filepattern = (
 	("All files","*.*"),
-	("MPEG 1 Layer 3 files","*.mp3"),
-	("Ogg Vorbis files","*.ogg"),
-	("iTunes AAC files","*.m4a"),
-	("WAVEform audio format","*.wav"),
-	("Advanced Audio Coding","*.aac"),
-	("Free Lossless Audio Codec","*.flac"),
-	("Audio Codec 3","*.ac3")
+	("MP3","*.mp3"),
+	("Ogg Vorbis","*.ogg"),
+	("iTunes AAC ","*.m4a"),
+	("Windows WAV","*.wav"),
+	("AAC","*.aac"),
+	("FLAC","*.flac"),
+	("AC3","*.ac3")
 )
 
+def beautify_uri(uri):
+	uri = unquote_filename(uri)
+	if uri.startswith("file://"):
+		return uri[7:]
+	return uri
+
 def vfs_walk(uri):
 	"""similar to os.path.walk, but with gnomevfs.
 	
@@ -137,7 +181,7 @@ def vfs_walk(uri):
 	if str(uri)[-1] != '/':
 		uri = uri.append_string("/")
 
-	filelist = []	
+	filelist = []  
 
 	try:
 		dirlist = gnomevfs.open_directory(uri)
@@ -177,13 +221,16 @@ def vfs_makedirs(path_to_create):
 			pass
 		except :
 			return False
-	return True	
+	return True  
 
 def vfs_unlink(filename):
 	gnomevfs.unlink(gnomevfs.URI(filename))
 
 def vfs_exists(filename):
-	return gnomevfs.exists(filename)
+	try:
+		return gnomevfs.exists(filename)
+	except:
+		return False
 
 # GStreamer gnomevfssrc helpers
 
@@ -199,29 +246,32 @@ def file_encode_filename(filename):
 def unquote_filename(filename):
 
 	f= urllib.unquote(filename)
-	try:
+	'''try:
 		# files are normaly in utf-8 ?
 		f = unicode(f, "utf-8")
 	except UnicodeDecodeError:
 		# but sometimes they are badly encoded, this is a failback
 		f = unicode(f, "latin1", "replace")
+	'''
 	return f
 
 
 use_gnomevfs = False
 
+
+def format_tag(tag):
+	if isinstance(tag, list):
+		if len(tag) > 1:
+			tag = ", ".join(tag[:-1]) + " & " + tag[-1]
+		else:
+			tag = tag[0]
+			
+	return tag
+
 def markup_escape(message):
-	if not isinstance(message, basestring):
-		print "markup error: '%s'" % message
-		import pdb
-		pdb.set_trace()
+	return gobject.markup_escape_text(message)
 
-	message = "&amp;".join(message.split("&"))
-	message = "&lt;".join(message.split("<"))
-    	message = "&gt;".join(message.split(">"))
-    	return message
-
-def filename_escape(str):
+def __filename_escape(str):
 	str = str.replace("'","\'")
 	str = str.replace("\"","\\\"")
 	str = str.replace("!","\!")
@@ -247,13 +297,13 @@ else:
 
 
 encoders = ( 
-	("flacenc",   "FLAC"), 
-	("wavenc",    "WAV"),
+	("flacenc",		"FLAC"), 
+	("wavenc",		"WAV"),
 	("vorbisenc", "Ogg Vorbis"),
-	("oggmux",    "Ogg Vorbis"),
-	("id3v2mux",  "MP3 Tags"),
-	("xingmux",   ""),
-	("lame",      "MP3"))
+	("oggmux",		"Ogg Vorbis"),
+	("id3v2mux",	"MP3 Tags"),
+	("xingmux",		""),
+	("lame",			"MP3"))
 
 for encoder, name in encoders:
 	have_it = True
@@ -306,7 +356,13 @@ class SoundConverterException(Exception):
 		
 
 def filename_to_uri(filename):
-	return "file://" + urllib.quote(os.path.abspath(filename))
+	"""Convert a filename to a valid uri.
+	Filename can be a relative or absolute path, or an uri.
+	"""
+	if vfs_exists(filename):
+		return str(gnomevfs.URI(filename))
+	else:
+		return "file://" + urllib.quote(os.path.abspath(filename))
 
 
 class SoundFile:
@@ -317,25 +373,27 @@ class SoundFile:
 	def __init__(self, uri, base_path=None):
 
 		self.uri = uri
+
 		if base_path:
 			self.base_path = base_path
 			self.filename = uri[len(base_path):]
 		else:
 			self.base_path, self.filename = os.path.split(self.uri)
 			self.base_path += "/"
-		self.filename_for_display = unquote_filename(self.filename)
+		self.filename_for_display = gobject.filename_display_name(
+				unquote_filename(self.filename))
 	
 		self.tags = {
 			"track-number": 0,
-			"title":  "Unknown Title",
+			"title":	"Unknown Title",
 			"artist": "Unknown Artist",
-			"album":  "Unknown Album",
+			"album":	"Unknown Album",
 		}
 		self.have_tags = False
 		self.tags_read = False
-		self.duration = 0	
+		self.duration = 0  
 		self.mime_type = None 
-	  
+		
 	def get_uri(self):
 		return self.uri
 		
@@ -365,14 +423,6 @@ class SoundFile:
 		return self.tags.keys()
 
 
-class TargetNameCreationFailure(SoundConverterException):
-
-	"""Exception thrown when TargetNameGenerator can't create name."""
-
-	def __init__(self, name):
-		SoundConverterException.__init__(self, _("File exists."),
-										 _("The file %s exists already"))
-
 class TargetNameGenerator:
 
 	"""Generator for creating the target name from an input name."""
@@ -392,7 +442,7 @@ class TargetNameGenerator:
 		else:
 			self.exists = os.path.exists
 
-	# This is useful for unit testing.		  
+	# This is useful for unit testing.			
 	def set_exists(self, exists):
 		self.exists = exists
 
@@ -411,16 +461,28 @@ class TargetNameGenerator:
 	def set_replace_messy_chars(self, yes_or_no):
 		self.replace_messy_chars = yes_or_no
 		
+	def _unicode_to_ascii(self, unicode_string):
+		# thanks to http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/251871
+		try:
+			unicode_string = unicode(unicode_string, "utf-8")
+			return unicodedata.normalize('NFKD', unicode_string).encode('ASCII', 'ignore')
+		except UnicodeDecodeError:
+			unicode_string = unicode(unicode_string, "iso-8859-1")
+			return unicodedata.normalize('NFKD', unicode_string).encode('ASCII', 'replace')
+			
+	
 	def get_target_name(self, sound_file):
 
+		assert self.suffix, "you just forgot to call set_target_suffix()"
+
 		u = gnomevfs.URI(sound_file.get_uri())
 		root, ext = os.path.splitext(u.path)
 		if u.host_port:
 			host = "%s:%s" % (u.host_name, u.host_port)
 		else:
 			host = u.host_name
-
-		root = urlparse.urlsplit(sound_file.get_base_path())[2]
+			
+		root = sound_file.get_base_path()
 		basename, ext = os.path.splitext(urllib.unquote(sound_file.get_filename()))
 		
 		dict = {
@@ -430,13 +492,20 @@ class TargetNameGenerator:
 			"artist": "",
 			"title": "",
 			"track-number": 0,
+			"track-count": 0,
+			"genre": "",
+			"year": "",
+			"date": "",
 		}
 		for key in sound_file.keys():
 			dict[key] = sound_file[key]
 		
 		pattern = os.path.join(self.subfolders, self.basename + self.suffix)
 		result = pattern % dict
+		if isinstance(result, unicode):
+			result = result.encode('utf-8')
 		if self.replace_messy_chars:
+			result = self._unicode_to_ascii(result)
 			s = ""
 			for c in result:
 				if c not in self.nice_chars:
@@ -451,13 +520,7 @@ class TargetNameGenerator:
 			folder = self.folder
 		result = os.path.join(folder, urllib.quote(result))
 
-		tuple = (u.scheme, host, result, "", u.fragment_identifier)
-		u2 = urlparse.urlunsplit(tuple)
-		#TODO:
-		#if self.exists(u2):
-		#	vfs_unlink(u2)
-		#	raise TargetNameCreationFailure(u2)
-		return u2
+		return result
 
 
 class ErrorDialog:
@@ -475,7 +538,7 @@ class ErrorDialog:
 
 	def show_exception(self, exception):
 		self.show("<b>%s</b>" % markup_escape(exception.primary),
-				  exception.secondary)
+					exception.secondary)
 
 
 class ErrorPrinter:
@@ -714,12 +777,12 @@ class Pipeline(BackgroundTask):
 	
 	def work(self):
 		#if self.pipeline.get_state() == gst.STATE_NULL:
-		#	log("error: pipeline.state == null")
+		#  log("error: pipeline.state == null")
 			#return False
 		#print "work:", self
 		#time.sleep(0.01)
 		if self.eos:
-			#print "  got eos:", self.sound_file.get_filename_for_display()
+			#print "	got eos:", self.sound_file.get_filename_for_display()
 			return False
 		return True
 
@@ -750,14 +813,14 @@ class Pipeline(BackgroundTask):
 			err, debug = message.parse_error()
 			self.eos = True
 			self.error = err
-			log("error: %s (%s)" % (err,
-				self.sound_file.get_filename_for_display()))
-			error.show("GStreamer Error", "%s\nfile: '%s'" % (err, 
-				self.sound_file.get_filename_for_display()))
+			#log("error: %s (%s)" % (err,
+			#	self.sound_file.get_filename_for_display()))
+			#error.show("GStreamer Error", "%s\nfile: '%s'" % (err, 
+			#	self.sound_file.get_filename_for_display()))
 		elif t == gst.MESSAGE_EOS:
 			self.eos = True
 		elif t == gst.MESSAGE_TAG:
-			self.found_tag(self, "", message.parse_tag())	
+			self.found_tag(self, "", message.parse_tag())  
 		return True
 
 	def play(self):
@@ -806,6 +869,7 @@ class TypeFinder(Pipeline):
 		mime_type = caps.to_string()
 		#debug("have_type:", mime_type, self.sound_file.get_filename_for_display())
 		self.sound_file.mime_type = None
+		#self.sound_file.mime_type = mime_type
 		for t in mime_whitelist:
 			if t in mime_type:
 				self.sound_file.mime_type = mime_type
@@ -881,7 +945,7 @@ class Decoder(Pipeline):
 	def get_duration(self):
 		""" return the total duration of the sound file """
 		#if not self.pipeline:
-		#	return 0
+		#  return 0
 		self.query_duration()
 		return self.sound_file.duration
 	
@@ -905,24 +969,17 @@ class TagReader(Decoder):
 
 
 	def found_tag(self, decoder, something, taglist):
-
-		debug("found_tags:", self.sound_file.get_filename_for_display())
+		#debug("found_tags:", self.sound_file.get_filename_for_display())
 		#debug("\ttitle=%s" % (taglist["title"]))
-		for k in taglist.keys():
+		"""for k in taglist.keys():
 			debug("\t%s=%s" % (k, taglist[k]))
+			if isinstance(taglist[k], gst.Date):
+				taglist["year"] = taglist[k].year
+				taglist["date"] = "%04d-%02d-%02d" % (taglist[k].year,
+									taglist[k].month, taglist[k].day)"""
+			
 		self.sound_file.add_tags(taglist)
 
-		# tags from ogg vorbis files comes with two callbacks,
-		# the first callback containing just the stream serial number.
-		# The second callback contains the tags we're interested in.
-		#if "serial" in taglist.keys():
-		#	print "Error, serial tag in ogg/vorbis, tags will be lost"
-		#	return
-		
-		# we want something useful in tags
-		#if "title" not in taglist.keys():
-		#	return
-
 		#self.found_tags = True
 		self.sound_file.have_tags = True
 
@@ -933,13 +990,11 @@ class TagReader(Decoder):
 
 	def work(self):
 		if not self.run_start_time:
-			self.run_start_time = time.time()
 			if self.sound_file.mime_type in tag_blacklist:
 				log("%s: type is %s, tag reading blacklisted" % (self.sound_file.get_filename_for_display(), self.sound_file.mime_type))
 				return False
-
+			self.run_start_time = time.time()
 		if time.time()-self.run_start_time > 5:
-			#log("TagReader timeout:", self.sound_file.get_filename_for_display())
 			# stop looking for tags after 5s 
 			return False
 		return Decoder.work(self) and not self.found_tags
@@ -979,8 +1034,8 @@ class Converter(Decoder):
 		self.delete_original = delete_original
 
 	#def setup(self):
-	#	self.init()
-	#	self.play()
+	#  self.init()
+	#  self.play()
 
 	def init(self):
 		#print "Converter.init()"
@@ -1136,7 +1191,7 @@ class FileList:
 			self.widget.append_column(column)
 	
 	def drag_data_received(self, widget, context, x, y, selection, 
-						   mime_id, time):
+							 mime_id, time):
 
 		if mime_id >= 0 and mime_id < len(self.drop_mime_types):
 			file_list = []
@@ -1179,7 +1234,7 @@ class FileList:
 		if not self.tagreaders.is_running():
 			self.tagreaders.run()
 	
-	def add_uris(self, uris, base=None):
+	def add_uris(self, uris, base=None, filter=None):
 
 		files = []
 
@@ -1187,15 +1242,18 @@ class FileList:
 			try:
 				info = gnomevfs.get_file_info(gnomevfs.URI(uri))
 			except gnomevfs.NotFoundError:
+				log('uri not found: \'%s\'' % uri)
 				continue
 			except gnomevfs.InvalidURIError:
+				log('unvalid uri: \'%s\'' % uri)
 				continue
 
 			if info.type == gnomevfs.FILE_TYPE_DIRECTORY:
 				filelist = vfs_walk(gnomevfs.URI(uri))
+				if filter:
+					filelist = [f for f in filelist if f.lower().endswith(filter)]
 				
 				for f in filelist:
-					#f = f[len(base)+1:]
 					files.append(f)
 			else:
 				files.append(uri)
@@ -1208,7 +1266,6 @@ class FileList:
 			if sound_file.get_uri() in self.filelist:
 				log(_("file already present: '%s'") % sound_file.get_uri())
 				continue 
-			#print "adding: '%s'" % sound_file.get_filename_for_display()
 			self.filelist[sound_file.get_uri()] = True
 
 			typefinder = TypeFinder(sound_file)
@@ -1224,7 +1281,7 @@ class FileList:
 
 	def format_cell(self, sound_file):
 		
-		template_tags    = "%(artist)s - <i>%(album)s</i> - <b>%(title)s</b>\n<small>%(filename)s</small>"
+		template_tags		 = "%(artist)s - <i>%(album)s</i> - <b>%(title)s</b>\n<small>%(filename)s</small>"
 		template_loading = "<i>%s</i>\n<small>%%(filename)s</small>" \
 							% _("loading tags...")
 		template_notags  = '<span foreground="red">%s</span>\n<small>%%(filename)s</small>' \
@@ -1233,7 +1290,7 @@ class FileList:
 		params = {}
 		params["filename"] = markup_escape(unquote_filename(sound_file.get_filename()))
 		for item in ("title", "artist", "album"):
-			params[item] = markup_escape(sound_file.get_tag(item))
+			params[item] = markup_escape(format_tag(sound_file.get_tag(item)))
 		if sound_file["bitrate"]:
 			params["bitrate"] = ", %s kbps" % (sound_file["bitrate"] / 1000)
 		else:
@@ -1254,11 +1311,7 @@ class FileList:
 
 	def append_file(self, sound_file):
 
-		#print "+", sound_file.get_filename_for_display()
-		iter = self.model.append()
-		sound_file.model = iter
-		self.model.set(iter, 0, self.format_cell(sound_file))
-		self.model.set(iter, 1, sound_file)
+		iter = self.model.append([self.format_cell(sound_file), sound_file])
 		self.window.progressbar.pulse()
 			
 	
@@ -1271,7 +1324,9 @@ class FileList:
 		fields["META"] = sound_file
 		fields["filename"] = sound_file.get_filename_for_display()
 
-		self.model.set(sound_file.model, 0, self.format_cell(sound_file))
+		for i in self.model:
+			if i[1] == sound_file:
+				i[0] = self.format_cell(sound_file)
 		self.window.set_sensitive()
 		self.window.progressbar.pulse()
 
@@ -1304,6 +1359,7 @@ class PreferencesDialog:
 	subfolder_patterns = [
 		("%(artist)s/%(album)s", _("artist/album")),
 		("%(artist)s-%(album)s", _("artist-album")),
+		("%(artist)s - %(album)s", _("artist - album")),
 	]
 	
 	defaults = {
@@ -1325,7 +1381,7 @@ class PreferencesDialog:
 	}
 
 	sensitive_names = ["vorbis_quality", "choose_folder", "create_subfolders",
-					   "subfolder_pattern"]
+						 "subfolder_pattern"]
 
 	def __init__(self, glade):
 		self.gconf = gconf.client_get_default()
@@ -1351,6 +1407,12 @@ class PreferencesDialog:
 		self.set_widget_initial_values(glade)
 		self.set_sensitive()
 
+		tips = gtk.Tooltips()
+		tip = _("Available patterns:")
+		for k in locale_patterns_dict.values():
+			tip += "\n" + k
+		tips.set_tip(self.custom_filename, tip)
+
 
 	def convert_setting_from_old_version(self):
 		""" try to convert previous settings"""
@@ -1374,8 +1436,8 @@ class PreferencesDialog:
 			w = glade.get_widget("into_selected_folder")
 		w.set_active(True)
 		
-		self.target_folder_chooser.set_filename(
-			self.get_string("selected-folder"))
+		uri = filename_to_uri(self.get_string("selected-folder"))
+		self.target_folder_chooser.set_uri(uri)
 		self.update_selected_folder()
 	
 		w = glade.get_widget("create_subfolders")
@@ -1398,10 +1460,10 @@ class PreferencesDialog:
 
 		mime_type = self.get_string("output-mime-type")
 
-		widgets = {		"audio/x-vorbis": (have_vorbisenc, "output_mime_type_ogg_vorbis"),
-						"audio/x-flac"  : (have_flacenc, "output_mime_type_flac"),
-						"audio/x-wav"   : (have_wavenc, "output_mime_type_wav"),
-						"audio/mpeg"    : (have_lame, "output_mime_type_mp3"),
+		widgets = {		 "audio/x-vorbis": (have_vorbisenc, "output_mime_type_ogg_vorbis"),
+						"audio/x-flac"	: (have_flacenc, "output_mime_type_flac"),
+						"audio/x-wav"		: (have_wavenc, "output_mime_type_wav"),
+						"audio/mpeg"		: (have_lame, "output_mime_type_mp3"),
 					}
 
 		# desactivate output if encoder plugin is not present
@@ -1419,7 +1481,7 @@ class PreferencesDialog:
 			w.show()
 		
 		widget_name = widgets.get(mime_type, None)[1]
-		print widget_name
+
 		if widget_name:
 			w = glade.get_widget(widget_name)
 			w.set_active(True)
@@ -1457,7 +1519,7 @@ class PreferencesDialog:
 
 	def update_selected_folder(self):
 		self.into_selected_folder.set_label(_("Into folder %s") % 
-										self.get_string("selected-folder"))
+			beautify_uri(self.get_string("selected-folder")))
 
 
 	def get_bitrate_from_settings(self):
@@ -1499,15 +1561,41 @@ class PreferencesDialog:
 	def update_example(self):
 		sound_file = SoundFile(os.path.expanduser("~/foo/bar.flac"))
 		sound_file.add_tags({
-			"artist": "<b>{Artist}</b>", 
-			"title": "<b>{Title}</b>", 
-			"album": "<b>{Album}</b>",
 			"track-number": 1L,
-			"track-count": 11L,
+			"track-count": 99L,
 		})
-		self.example.set_markup(self.generate_filename(sound_file, for_display=True))
+		sound_file.add_tags(locale_patterns_dict)
+
+		s = markup_escape(self.generate_filename(sound_file, for_display=True))
+		p = 0
+		replaces = []
+
+		while 1:
+			b = s.find('{', p)
+			if b == -1:
+				break
+			e = s.find('}',b)
+			
+			tag = s[b:e+1]
+			if tag.lower() in [v.lower() for v in locale_patterns_dict.values()]:
+				k = tag
+				l = k.replace("{","<b>{")
+				l = l.replace("}","}</b>")
+				replaces.append([k,l])
+			else:
+				k = tag
+				l = k.replace("{","<span foreground=\"red\"><i>{")
+				l = l.replace("}","}</i></span>")
+				replaces.append([k,l])
+			p = b+1
+			
+		for k,l in replaces:
+			s = s.replace(k, l)
+
+		self.example.set_markup(s)
 		
-		markup = _("<small>Target bitrate: %s</small>") % self.get_bitrate_from_settings()
+		markup = "<small>%s</small>" % (_("Target bitrate: %s") % 
+					self.get_bitrate_from_settings())
 		self.aprox_bitrate.set_markup( markup )
 
 	def generate_filename(self, sound_file, for_display=False):
@@ -1521,16 +1609,10 @@ class PreferencesDialog:
 					}.get(output_type, None)
 
 		generator = TargetNameGenerator()
+
 		generator.set_target_suffix(output_suffix)
-		if self.get_int("same-folder-as-input"):
-			tuple = urlparse.urlparse(sound_file.get_uri())
-			path = tuple[2]
-			generator.set_folder(os.path.dirname(path))
-		else:
-			path, filename = os.path.split(sound_file.get_filename())
-			path = ""
-		
-			generator.set_folder(os.path.join(self.get_string("selected-folder"), path))
+		if not self.get_int("same-folder-as-input"):
+			generator.set_folder(self.get_string("selected-folder"))
 			if self.get_int("create-subfolders"):
 				generator.set_subfolder_pattern(
 					self.get_subfolder_pattern())
@@ -1544,12 +1626,10 @@ class PreferencesDialog:
 			return generator.get_target_name(sound_file)
 	
 	def process_custom_pattern(self, pattern):
-			pattern = pattern.replace("{Artist}", "%(artist)s")
-			pattern = pattern.replace("{Album}", "%(album)s")
-			pattern = pattern.replace("{Title}", "%(title)s")
-			pattern = pattern.replace("{Track}", "%(track-number)02d")
-			pattern = pattern.replace("{Total}", "%(track-total)02d")
-			return pattern
+		
+		for k in custom_patterns:
+			pattern = pattern.replace(k, custom_patterns[k])
+		return pattern
 
 	def set_sensitive(self):
 	
@@ -1621,7 +1701,7 @@ class PreferencesDialog:
 		ret = self.target_folder_chooser.run()
 		self.target_folder_chooser.hide()
 		if ret == gtk.RESPONSE_OK:
-			folder = self.target_folder_chooser.get_filename()
+			folder = self.target_folder_chooser.get_uri()
 			if folder:
 				self.set_string("selected-folder", folder)
 				self.update_selected_folder()
@@ -1721,7 +1801,7 @@ class PreferencesDialog:
 		quality_to_preset = {
 			"cbr": {64:0, 96:1, 128:2, 192:3, 256:4},
 			"abr": {64:0, 96:1, 128:2, 192:3, 256:4},
-			"vbr": {9:0,   7:1,	  5:2,	 3:3,	1:4}, # inverted !
+			"vbr": {9:0,	 7:1,		 5:2,		3:3,	1:4}, # inverted !
 		}
 			
 		if quality in quality_to_preset[mode]:
@@ -1788,7 +1868,12 @@ class ConverterQueue(TaskQueue):
 		except gnomevfs.InvalidURIError:
 			log("Invalid URI: '%s'" % output_filename)
 			return
-				
+		
+		# do not overwrite source file !!
+		if output_filename == sound_file.get_uri():
+			error.show(_("Cannot overwrite source file(s)!"), "")
+			raise ConverterQueueCanceled()
+		
 		if exists:
 			if self.overwrite_action != None:
 				result = self.overwrite_action
@@ -1832,8 +1917,8 @@ class ConverterQueue(TaskQueue):
 				raise ConverterQueueCanceled()
 			
 		c = Converter(sound_file, output_filename, 
-					  self.window.prefs.get_string("output-mime-type"),
-					  self.window.prefs.get_int("delete-original"))
+						self.window.prefs.get_string("output-mime-type"),
+						self.window.prefs.get_int("delete-original"))
 		c.set_vorbis_quality(self.window.prefs.get_float("vorbis-quality"))
 		
 		quality = {
@@ -1889,7 +1974,7 @@ class ConverterQueue(TaskQueue):
 		self.window.conversion_ended()
 		total_time = self.run_finish_time - self.run_start_time
 		self.window.set_status(_("Conversion done, in %s") % 
-							   self.format_time(total_time))
+								 self.format_time(total_time))
 
 	def format_time(self, seconds):
 		units = [(86400, "d"),
@@ -1942,8 +2027,8 @@ class CustomFileChooser:
 		self.combo.add_attribute(combo_rend, 'text', 0)
 	
 		# get all (gstreamer) knew files Todo
-		for files in filepattern:
-			self.add_pattern(files[0],files[1])
+		for name, pattern in filepattern:
+			self.add_pattern(name,pattern)
 		self.combo.set_active(0)
 		
 	def add_pattern(self,name,pat):
@@ -1957,13 +2042,22 @@ class CustomFileChooser:
 		self.pattern.append(pat)
 		self.store.append(["%s (%s)" %(name,pat)])
 		
+
+	def filter_cb(self, info, pattern):
+		filename = info[2]
+		return filename.lower().endswith(pattern[1:])
+
 	def on_combo_changed(self,w):
 		"""
 		Callback for combobox "changed" signal\n
 		Set a new filter for the filechooserwidget
 		"""
 		filter = gtk.FileFilter()
-		filter.add_pattern(self.pattern[self.combo.get_active()])
+		active = self.combo.get_active()
+		if active:
+			filter.add_custom(gtk.FILE_FILTER_DISPLAY_NAME, self.filter_cb, self.pattern[self.combo.get_active()])
+		else:
+			filter.add_pattern('*.*')
 		self.fcw.set_filter(filter)
 		
 	def run(self):
@@ -1989,8 +2083,8 @@ class SoundConverterWindow:
 
 	"""Main application class."""
 
-	sensitive_names = [ "remove", "convert_button" ]
-	unsensitive_when_converting = [ "remove", "prefs_button" ,"toolbutton_addfile", "toolbutton_addfolder", "filelist", "menubar" ]
+	sensitive_names = [ "remove", "clear", "toolbutton_clearlist", "convert_button" ]
+	unsensitive_when_converting = [ "remove", "clear", "prefs_button" ,"toolbutton_addfile", "toolbutton_addfolder", "toolbutton_clearlist", "filelist", "menubar" ]
 
 	def __init__(self, glade):
 	
@@ -2022,6 +2116,22 @@ class SoundConverterWindow:
 		self.addfolderchooser.set_select_multiple(True)
 		self.addfolderchooser.set_local_only(not use_gnomevfs)
 
+
+		self.combo = gtk.ComboBox()
+		#self.combo.connect("changed",self.on_combo_changed)
+		self.store = gtk.ListStore(str)
+		self.combo.set_model(self.store)
+		combo_rend = gtk.CellRendererText()
+		self.combo.pack_start(combo_rend, True)
+		self.combo.add_attribute(combo_rend, 'text', 0)
+	
+		# get all (gstreamer) knew files Todo
+		for files in filepattern:
+			self.store.append(["%s (%s)" %(files[0],files[1])])
+
+		self.combo.set_active(0)
+		self.addfolderchooser.set_extra_widget(self.combo)
+
 		self.connect(glade, [self.prefs])
 		
 		self.about.set_property("name", NAME)
@@ -2069,10 +2179,6 @@ class SoundConverterWindow:
 		ret = self.addchooser.run()
 		self.addchooser.hide()
 		if ret == gtk.RESPONSE_OK:
-			files = []
-			#for uri in self.addchooser.get_uris():
-			#	files.append(SoundFile(uri))
-			#self.filelist.add_files(files)
 			self.filelist.add_uris(self.addchooser.get_uris())
 		self.set_sensitive()
 
@@ -2081,18 +2187,25 @@ class SoundConverterWindow:
 		ret = self.addfolderchooser.run()
 		self.addfolderchooser.hide()
 		if ret == gtk.RESPONSE_OK:
+			
 			folders = self.addfolderchooser.get_uris()
 			
-			self.filelist.add_uris(folders)
+			filter = None
+			if self.combo.get_active():
+				filter = os.path.splitext(filepattern[self.combo.get_active()] 
+						[1]) [1]
+
+			
+			self.filelist.add_uris(folders, filter = filter)
 
 			#base,notused = os.path.split(os.path.commonprefix(folders))
 			#filelist = []
 			#files = []
 			#for folder in folders:
-			#	filelist.extend(vfs_walk(gnomevfs.URI(folder)))
+			#  filelist.extend(vfs_walk(gnomevfs.URI(folder)))
 			#for f in filelist:
-			#	f = f[len(base)+1:]
-			#	files.append(SoundFile(base+"/", f))
+			#  f = f[len(base)+1:]
+			#  files.append(SoundFile(base+"/", f))
 			#self.filelist.add_files(files)
 		self.set_sensitive()
 
@@ -2103,6 +2216,15 @@ class SoundConverterWindow:
 			self.filelist.remove(i)
 			model, paths = self.filelist_selection.get_selected_rows()
 		self.set_sensitive()
+		
+	def on_clearlist_activate(self, *args):
+		self.filelist_selection.select_all();
+		model, paths = self.filelist_selection.get_selected_rows()
+		while paths:
+			i = self.filelist.model.get_iter(paths[0])
+			self.filelist.remove(i)
+			model, paths = self.filelist_selection.get_selected_rows()
+		self.set_sensitive()
 
 	def do_convert(self):
 		try:
@@ -2110,6 +2232,8 @@ class SoundConverterWindow:
 				self.converter.add(sound_file)
 		except ConverterQueueCanceled:
 			log(_("canceling conversion."))
+			self.conversion_ended()
+			self.set_status(_("Conversion canceled"))
 		else:
 			self.set_status("")
 			self.converter.run()
@@ -2205,7 +2329,7 @@ class SoundConverterWindow:
 		self.set_widget_sensitive("remove", 
 			self.filelist_selection.count_selected_rows() > 0)
 		self.set_widget_sensitive("convert_button", 
-								  self.filelist.is_nonempty())
+									self.filelist.is_nonempty())
 
 		self._lock_convert_button = True
 		self.sensitive_widgets["convert_button"].set_active(
@@ -2228,7 +2352,7 @@ class SoundConverterWindow:
 		
 		self.set_status(_("Converting"))
 		
-		self.progressfile.set_markup("<i><small>%s</small></i>" % current_file)
+		self.progressfile.set_markup("<i><small>%s</small></i>" % markup_escape(current_file))
 		fraction = float(done_so_far) / total
 	
 		self.progressbar.set_fraction( min(fraction, 1.0) )
@@ -2281,7 +2405,7 @@ def cli_tags_main(input_files):
 			keys = input_file.keys()
 			keys.sort()
 			for key in keys:
-				print "	 %s: %s" % (key, input_file[key])
+				print "		%s: %s" % (key, input_file[key])
 
 
 class CliProgress:
@@ -2368,11 +2492,11 @@ def print_help(*args):
 	for short_arg, long_arg, func, doc in options:
 		print
 		if short_arg[-1] == ":":
-			print "	 -%s arg, --%sarg" % (short_arg[:1], long_arg)
+			print "		-%s arg, --%sarg" % (short_arg[:1], long_arg)
 		else:
-			print "	 -%s, --%s" % (short_arg[:1], long_arg)
+			print "		-%s, --%s" % (short_arg[:1], long_arg)
 		for line in textwrap.wrap(doc):
-			print "	   %s" % line
+			print "			%s" % line
 	sys.exit(0)
 
 
@@ -2421,9 +2545,9 @@ def main():
 		for key in settings:
 			print key, settings[key]
 		return
-	
-	args = map(filename_to_uri, args)
 
+	args = map(filename_to_uri, args)
+	
 	if get_option("mode") == "gui":
 		gui_main(args)
 	elif get_option("mode") == "tags":
